package org.springblade.modules.quartz.task;

import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springblade.common.enums.FacilityStatusEnum;
import org.springblade.modules.business.entity.Facility;
import org.springblade.modules.business.service.IFacilityLogService;
import org.springblade.modules.business.service.IFacilityService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Calendar;
import java.util.concurrent.TimeUnit;

//设备心跳信息处理
@Slf4j
@Component
public class VendingTimeTask {

    @Resource
    private RedissonClient redissonClient;
    @Autowired
    private IFacilityLogService facilityLogService;
    @Autowired
    private IFacilityService facilityService;

    /**
     * 离线校验
     */
    public void onlineTesting() {
        RLock lock = redissonClient.getLock("vending:online:testing");
        lock.lock(10, TimeUnit.SECONDS);
        try {
            //查询设备心跳，超过6分钟未发心跳则判断为离线（设备心跳为5分钟）
            Calendar instance = Calendar.getInstance();
            instance.set(Calendar.MINUTE, -6);

            facilityService.lambdaUpdate()
                    .lt(Facility::getHeartbeatTime, instance.getTime())
                    .eq(Facility::getFacilityStatus, FacilityStatusEnum.ON_LINE.getValue())
                    .set(Facility::getFacilityStatus, FacilityStatusEnum.OFF_LINE.getValue())
                    .update();

        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(), e);
        } finally {
            if (lock.isLocked()) {
                lock.unlock();
            }
        }
    }

}
